/* ======================================================================= */
/* DSP_minerror_d.c -- Minimum Energy Error Search                         */
/*                     Driver code implementation                          */
/*                                                                         */
/* Rev 0.0.1                                                               */
/*                                                                         */
/* Copyright (C) 2011 Texas Instruments Incorporated - http://www.ti.com/  */ 
/*                                                                         */
/*                                                                         */
/*  Redistribution and use in source and binary forms, with or without     */
/*  modification, are permitted provided that the following conditions     */
/*  are met:                                                               */
/*                                                                         */
/*    Redistributions of source code must retain the above copyright       */
/*    notice, this list of conditions and the following disclaimer.        */
/*                                                                         */
/*    Redistributions in binary form must reproduce the above copyright    */
/*    notice, this list of conditions and the following disclaimer in the  */
/*    documentation and/or other materials provided with the               */
/*    distribution.                                                        */
/*                                                                         */
/*    Neither the name of Texas Instruments Incorporated nor the names of  */
/*    its contributors may be used to endorse or promote products derived  */
/*    from this software without specific prior written permission.        */
/*                                                                         */
/*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS    */
/*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT      */
/*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR  */
/*  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT   */
/*  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,  */
/*  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT       */
/*  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,  */
/*  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY  */
/*  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT    */
/*  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE  */
/*  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.   */
/*                                                                         */
/* ======================================================================= */

#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <limits.h>
#include <c6x.h>

/* ======================================================================= */
/* Interface header files for the natural C and optimized C code           */
/* ======================================================================= */
#include "DSP_minerror_cn.h"
#include "DSP_minerror.h"

/* Defines */
#if defined(__TI_EABI__)
#define kernel_size _kernel_size
#endif

extern char kernel_size;

/* ======================================================================= */
/* Inform the compiler the arrays are double word alligned                 */
/* ======================================================================= */
#pragma DATA_ALIGN(GSP0_TABLE, 16);

/* ======================================================================= */
/* Parameters of fixed dataset                                             */
/* ======================================================================= */
#define NTABLE    (9*256)
#define NerrCoefs (10)
#define GSP0_TERMS 9
#define GSP0_NUM   256
#define PAD        16

/* ======================================================================= */
/* Initilized arrays with fixed test data                                  */
/* ======================================================================= */
const short GSP0_TABLE[GSP0_NUM*GSP0_TERMS + 2*PAD] =
{
    0xFFAA,  0x5AA3,  0x765A,  0x5AA5,  0xCDEA,  0x7BCD,  0xFFDC,  0x7FFF,      // Beginning
    0xFEAD,  0xDADE,  0xCEAD,  0xFDAC,  0xAAAA,  0xA5A5,  0x5A5A,  0x55AA,      // PAD

    0xFF66,  0xFFFE,  0x0164,  0x0009,  0xCC13,  0x0000,  0x0000,  0x045E,
    0xEADA,  0x0A2C,  0x057E,  0x0486,  0xE076,  0x0002,  0xFFFF,  0xF452,
    0x007D,  0x0002,  0x0002,  0x0003,  0xFFC5,  0x001F,  0xFD7F,  0xFFFE,
    0xFE2C,  0xFFFA,  0x000C,  0xFFFE,  0x00AB,  0x00EB,  0x0000,  0xFBBE,
    0x0000,  0x00DD,  0xFB44,  0xFFFC,  0x0008,  0x000D,  0x00AC,  0x0000,
    0xFFE4,  0xE48D,  0xFFFF,  0x0027,  0x0000,  0x0000,  0x0001,  0x3C30,
    0x0000,  0xFFFF,  0xFF26,  0xFFFC,  0x0A53,  0xFFFA,  0x0030,  0xFF5E,
    0xFEA7,  0x00EF,  0xFEF8,  0xFF55,  0xFE71,  0x01F2,  0xFE33,  0xFD31,
    0xFC72,  0xFE37,  0x03A4,  0x0001,  0xF97B,  0x0003,  0xFFFE,  0xFFFF,
    0xFFF5,  0x0003,  0xDA57,  0x53E8,  0xFFE9,  0x0007,  0xFFD6,  0x00A4,
    0x0000,  0x05C6,  0xFFF8,  0xFFFF,  0xFFFF,  0xFE85,  0xFFFD,  0xF629,
    0x0001,  0x0005,  0x35C1,  0xFF86,  0xC1DF,  0x009D,  0x0015,  0xFC00,
    0xFAD2,  0xFFFE,  0x006F,  0xE745,  0xFFD0,  0xFC23,  0xFD66,  0xFFFB,
    0x2CBB,  0x0017,  0xE51D,  0xFFF9,  0xE267,  0xFFFE,  0x02A7,  0xF9BC,
    0x3EBC,  0x15EA,  0x0D7C,  0xFF82,  0xFF2A,  0x0000,  0x00B5,  0xFFFF,
    0xFE70,  0xFFFE,  0x0D97,  0x0D96,  0xFFFE,  0xFFDE,  0x00E0,  0xF88B,
    0xFFFE,  0xF140,  0xFE88,  0x04BF,  0x4B97,  0xFFC4,  0xF936,  0xFCFC,
    0xFFF8,  0xF141,  0xFFFF,  0x0015,  0xFFFE,  0x1AA3,  0x0005,  0xFE62,
    0x0B3A,  0x0000,  0xFFFE,  0xFFF9,  0x9E99,  0x0027,  0xFE60,  0xFFFC,
    0xFFFC,  0xD895,  0x0159,  0xFF97,  0xFF85,  0xF5D2,  0xFB71,  0x03A3,
    0xF971,  0x0000,  0xF657,  0xBB75,  0x015C,  0xFFFF,  0x0000,  0x0018,
    0x0006,  0x0001,  0xFE12,  0xF921,  0x001E,  0xFFB3,  0x0002,  0xFFFF,
    0xFFFD,  0x0003,  0x006F,  0x01ED,  0xFF94,  0x0000,  0x0574,  0xFFFD,
    0x00CF,  0x004C,  0xFFFF,  0xE0F5,  0xFB5F,  0x0632,  0xFFFF,  0xFFC4,
    0xFDAE,  0xEF70,  0x0007,  0x0589,  0xE7F9,  0x9AE1,  0xFFFD,  0xFFFC,
    0x000B,  0x0000,  0xFF82,  0x059C,  0xFE96,  0xA45E,  0xF56D,  0xFFFF,
    0xFE8C,  0x03A0,  0xF128,  0xFCE8,  0xFFF7,  0x0000,  0xFFEA,  0xFFD5,
    0x0007,  0xFFFE,  0xFFEA,  0xE552,  0xFEAD,  0xFE83,  0xFFFE,  0x0065,
    0x01DB,  0x02E2,  0x0037,  0x051F,  0x0C6E,  0xFFC4,  0x0001,  0xFFFE,
    0x0773,  0x028D,  0x0001,  0xFFF2,  0xF41E,  0x08C9,  0x0059,  0xFFFE,
    0xFFF4,  0x0000,  0x000E,  0xFFFE,  0xFFBE,  0x0007,  0xFFF3,  0xFFEB,
    0x0048,  0x1FB1,  0xFB20,  0x01E1,  0x003D,  0xFFFD,  0xFFFE,  0x0003,
    0xE231,  0xE116,  0xFFFE,  0x0000,  0x07ED,  0xFFFF,  0x0002,  0x0001,
    0xF74C,  0xFFFA,  0xF100,  0xFFFC,  0x045F,  0xFFF0,  0x00E5,  0xF324,
    0x004C,  0xFFFD,  0x1B1E,  0xE5A2,  0xFD54,  0xFFA6,  0xFF78,  0x001E,
    0x0008,  0xF87E,  0xFFFF,  0xFFFE,  0x0001,  0x0000,  0xFFE8,  0x0027,
    0xFFFB,  0xBFA3,  0xFFE0,  0x071D,  0xE163,  0x0544,  0xFFED,  0x195F,
    0xFFFF,  0xFFFF,  0x0003,  0x0001,  0x0038,  0x0049,  0xFF06,  0xFFFE,
    0x003B,  0xFFFF,  0xFFF4,  0x008B,  0xEF2E,  0xFF8E,  0xFFF9,  0x020E,
    0xFFF8,  0x0000,  0x00EC,  0x17EB,  0x0014,  0x1261,  0x0ECC,  0x17F0,
    0x0073,  0xFFE5,  0x001F,  0x0713,  0xFF94,  0x0000,  0x41CA,  0xFFFF,
    0xFE2F,  0x00E5,  0x0008,  0x0509,  0xFFF1,  0xFD8A,  0x06C8,  0xFDE3,
    0xFFFA,  0xFF3B,  0x0024,  0xFFF7,  0xF353,  0xFFF4,  0x009F,  0x0D81,
    0x0486,  0x0407,  0xEA8E,  0x0083,  0x000A,  0xFFE4,  0xFFFC,  0x3562,
    0x0227,  0x44DE,  0xFFF3,  0xFEC6,  0x0002,  0xFFC6,  0xFFFF,  0x0681,
    0xFFFE,  0xFF5F,  0xFE14,  0xFF37,  0xFFE1,  0xFFFC,  0x0125,  0xFEE6,
    0x0018,  0xFFFC,  0x004C,  0x0001,  0xFEBC,  0xFEE5,  0x05BE,  0x01C1,
    0xD7C7,  0xFFFD,  0xFFFA,  0xD7FD,  0xFFFE,  0x0000,  0xFFF5,  0x4BF6,
    0xEBFE,  0x0035,  0x0003,  0xFFFE,  0xFCC1,  0xFFE4,  0x0001,  0x00BD,
    0x333B,  0x00DB,  0x57D6,  0x0000,  0x000E,  0xFFDD,  0x2446,  0x000D,
    0x0005,  0xFF7B,  0xCDFC,  0xFFF9,  0x0011,  0xFFC5,  0xFFFC,  0xFFFF,
    0x0380,  0xFF72,  0xFFFC,  0x0025,  0x364C,  0x77F5,  0xFFFF,  0x131B,
    0xFFFF,  0x0001,  0xFFFC,  0xFFF8,  0x001B,  0x2750,  0xFFFF,  0x1CD9,
    0xFF5D,  0x03F5,  0xFFFC,  0xFD02,  0x0000,  0xFFC8,  0x0072,  0xFD46,
    0x0027,  0x00DF,  0xFF08,  0xE255,  0x0001,  0x2733,  0xFEDE,  0xFFE2,
    0xFFFF,  0xFFBB,  0x0000,  0xFFFE,  0x0030,  0x0002,  0x0001,  0xFFFC,
    0x0246,  0x0009,  0xFFF8,  0x0FB1,  0xF9F3,  0x000E,  0x02C8,  0xFFD6,
    0x7555,  0x0416,  0x0002,  0x0000,  0xFA57,  0x00BE,  0xFF99,  0xFD25,
    0xFFFF,  0xFFFC,  0xCE61,  0x011D,  0xFFFD,  0x0000,  0xF9E0,  0xFFE9,
    0x03E5,  0xFFF2,  0xFFE1,  0x0002,  0x02F1,  0x01F8,  0x0027,  0xF0CE,
    0x7A3D,  0x1798,  0x0025,  0x0000,  0x0000,  0xFFFA,  0x19EC,  0x197F,
    0x0025,  0xFFFB,  0x7DB3,  0x427B,  0x0002,  0xFFFC,  0xF62A,  0xFFE1,
    0xFE1B,  0x0002,  0xFFFF,  0x0000,  0xFFEB,  0xF8F9,  0x0032,  0xFF93,
    0x1568,  0xFFFF,  0x0000,  0x0002,  0x0018,  0xFDA6,  0x00FD,  0x0036,
    0x04B3,  0x0001,  0xFFFB,  0xFE49,  0x0001,  0x00B2,  0xFFC2,  0xE5F9,
    0xFFF8,  0x0000,  0xFA7D,  0xFFCE,  0xFF89,  0xFFFE,  0xF6C3,  0xFFFF,
    0xFF94,  0xFFFF,  0x0280,  0x00DD,  0xFFFE,  0x00C3,  0xF7EC,  0xF75E,
    0xF2D6,  0xFD39,  0xFFFF,  0xFFFE,  0xFC15,  0xFFC0,  0x0025,  0x001A,
    0x006E,  0xFFD1,  0xFFF7,  0x0494,  0x0011,  0xF919,  0x11F2,  0x0741,
    0x35F5,  0x0253,  0x0005,  0xE546,  0x0003,  0xE740,  0xFFFE,  0xFF03,
    0x0004,  0x014E,  0x041F,  0x0A16,  0xFFF5,  0x0EC0,  0x0000,  0xFC40,
    0x086C,  0x0055,  0x23B8,  0xFFFE,  0xFFF2,  0xE1EE,  0x0000,  0xFC9E,
    0xFFFC,  0xFFF5,  0x1C21,  0xFC7E,  0x0126,  0xFFCA,  0xFFFB,  0x01B2,
    0x0000,  0x0F95,  0xFFCD,  0x0006,  0x0000,  0x00B9,  0x00AE,  0x0003,
    0xFFEC,  0xFFFD,  0xFBAE,  0xFFFD,  0x05C7,  0xED4E,  0xFFFE,  0x0003,
    0xFFFE,  0xFFFD,  0xFFFE,  0x2825,  0x0005,  0xFFFA,  0x0009,  0xFF55,
    0xFF90,  0x002C,  0xFF5C,  0xE178,  0x0021,  0x00BC,  0xFFFF,  0x0018,
    0x000B,  0x04E5,  0x001D,  0x0002,  0x0FDF,  0x3549,  0xFB2C,  0xCA86,
    0x1A7D,  0xE4C1,  0xFFFF,  0x0038,  0xFFF1,  0x0005,  0x0034,  0x0198,
    0x0001,  0xFF8A,  0xFFB0,  0xFFFC,  0xFFFF,  0xFFFF,  0xF626,  0x0EBD,
    0xF924,  0x0000,  0x001B,  0xFFF8,  0x0703,  0x01F0,  0xFFF4,  0xFFF0,
    0x0008,  0x0000,  0xFE92,  0x0ADD,  0xFF05,  0x0067,  0xFFEC,  0x011D,
    0xFFFF,  0xFFF4,  0x0020,  0xFFCB,  0x0BEC,  0xFF9F,  0xFF99,  0xFFCF,
    0xF1D9,  0x4CC1,  0x1264,  0x0002,  0xDCBE,  0xF984,  0x3B0D,  0x026F,
    0xFFF6,  0x3A06,  0xFBEA,  0xFFFD,  0x0018,  0xFFE7,  0x0030,  0x27F4,
    0x0001,  0x016E,  0xFF26,  0xFFFE,  0xFFFA,  0xFFFD,  0xFFFD,  0x1E68,
    0x000B,  0xFFFF,  0x0001,  0xFE44,  0xCB0A,  0xFFFE,  0xFFF7,  0x0002,
    0x1C77,  0xF3B2,  0x0000,  0x0013,  0xFFC5,  0xFFFE,  0xF543,  0xFFF7,
    0xFFFF,  0x0013,  0x02BA,  0xFFF8,  0x0002,  0x02BC,  0xFFFF,  0x01BB,
    0x09FA,  0x04FE,  0x001F,  0xFCB2,  0xE888,  0xFFFC,  0x0000,  0xFF9E,
    0xFFF9,  0x0006,  0x0003,  0xF3D4,  0x0007,  0xFFFF,  0xFFFA,  0x003A,
    0x0000,  0xFFFE,  0xFFFF,  0x0001,  0xF9F1,  0x048D,  0xFFD9,  0x0021,
    0x0001,  0x0C71,  0x2F65,  0x0072,  0x0004,  0xFFFF,  0x011A,  0xFCA1,
    0x011A,  0x0015,  0x0002,  0x0019,  0xFFF8,  0x0A31,  0xFFFF,  0x1AC7,
    0xFFFE,  0x083B,  0x09AF,  0x0024,  0xFFFB,  0x1B4D,  0xFFFC,  0x0010,
    0x09B9,  0x0219,  0xFF50,  0xFF81,  0xFD59,  0x0234,  0x003F,  0x0000,
    0x008F,  0xFFFF,  0xFE52,  0x0080,  0x001D,  0x0000,  0x0391,  0x0001,
    0xA1AA,  0xFBAA,  0x1ED3,  0xFFFE,  0xFFF9,  0x1F99,  0x017C,  0xFFF5,
    0x0006,  0xD0A3,  0x733A,  0x0006,  0x0620,  0x0399,  0xC22C,  0xF629,
    0x00EA,  0xFFFF,  0x8F5B,  0x0017,  0x7A28,  0xFEF6,  0xFFFD,  0xFF9D,
    0x0016,  0x02D4,  0x0032,  0xFF37,  0xFFF3,  0xFF34,  0x2CB0,  0xFF2D,
    0x0000,  0x000C,  0xFF41,  0xF427,  0x1642,  0x0002,  0xFFFF,  0xEB0C,
    0xFFD0,  0xE52D,  0x4AC2,  0x001E,  0xFCC1,  0x4A7B,  0xECB3,  0xFFDB,
    0x002F,  0xF272,  0x0040,  0x03A4,  0x0001,  0x0003,  0xFFE1,  0xF862,
    0x1EC3,  0xFB78,  0x0000,  0xF49D,  0x02EB,  0xFFFE,  0x000E,  0xFCE4,
    0xFFFF,  0xE600,  0x0A6F,  0xFFFF,  0xFFF8,  0xFFFB,  0xFFFF,  0xFC04,
    0x0001,  0x0026,  0xFFD6,  0x00DE,  0xFFD9,  0xFF09,  0xFFD3,  0x001C,
    0x0001,  0xFFF2,  0xA540,  0x02EE,  0x001F,  0xFFE3,  0x0003,  0x05F6,
    0xFFF7,  0x1EA3,  0x0000,  0xFCAE,  0xFF00,  0xFFF9,  0xFFEE,  0xFFFF,
    0x000D,  0x0C73,  0xFEA0,  0x024D,  0xFFC1,  0x0000,  0xFFEC,  0x0004,
    0x9255,  0x0008,  0x0900,  0x0000,  0x0056,  0x001A,  0xE9E1,  0x0116,
    0xF702,  0xFF8E,  0xFFE9,  0xFEFF,  0x002F,  0xFFFF,  0xFFF5,  0x009C,
    0x00DC,  0x8701,  0xFFE8,  0xFEF3,  0xF77A,  0x0006,  0x00AD,  0x0003,
    0x0000,  0x0000,  0x0052,  0x0019,  0x0000,  0xFFFF,  0xFF7B,  0x1B8B,
    0xFFFF,  0x11A6,  0xFFE9,  0xFFBE,  0xC35A,  0x0000,  0xFF23,  0xF5F9,
    0x001C,  0x0007,  0xFAA2,  0x0000,  0xFE74,  0xFFFE,  0xFFFF,  0xFFCF,
    0xFFEF,  0xFFFE,  0x0025,  0x0000,  0xFFFE,  0xFF74,  0x0000,  0xC26B,
    0xFFFF,  0xF814,  0x0004,  0xFE77,  0x0000,  0x0025,  0x0001,  0x000B,
    0x0000,  0xFDE7,  0xF0A0,  0x00A7,  0x0003,  0xFFF2,  0x2053,  0x0001,
    0x0003,  0x0042,  0xF6D3,  0xFFF2,  0xFB3F,  0x0019,  0xFFF9,  0xEA0D,
    0x001C,  0x1863,  0x1849,  0x04E7,  0x0010,  0xE948,  0x0000,  0xFFFF,
    0xFFFF,  0xF176,  0x5F8A,  0x0000,  0x0001,  0x0019,  0xFE4C,  0x0322,
    0xFFE8,  0x00C5,  0x1C9A,  0x0000,  0xFAC4,  0xDE12,  0x099E,  0x0001,
    0xFFDD,  0x0000,  0x001C,  0xFC66,  0xF152,  0xFFF1,  0x0005,  0x003D,
    0xFFFF,  0xF8AC,  0xFEA6,  0x00B8,  0xFFFF,  0x009B,  0x0012,  0xFFFF,
    0xFFD8,  0xFE5D,  0xE84B,  0x000A,  0xE287,  0xFFD5,  0xF884,  0x0010,
    0xFFF9,  0x0002,  0xFFEC,  0x0002,  0x0002,  0x1E8A,  0xFA99,  0xFFAE,
    0x0000,  0x0185,  0xFFF2,  0xFFF2,  0xFFFF,  0xFFB4,  0xFFF2,  0x0006,
    0x03F3,  0xFFFE,  0x002C,  0x0696,  0x02A8,  0x0003,  0xFE9B,  0x0000,
    0xFFFF,  0x0069,  0xFD6E,  0xFFFE,  0xFFFF,  0xFFCC,  0x02A1,  0xFD29,
    0x09F8,  0xFEEF,  0xFFFE,  0xFFFF,  0xFA15,  0xFFFC,  0x0180,  0xFFD0,
    0xFFFE,  0xF3CA,  0xE42D,  0xFFFE,  0xFFFB,  0x003C,  0x0000,  0x0000,
    0x0003,  0x042A,  0x0FB0,  0x0000,  0xEFDB,  0x0024,  0xF36A,  0x0009,
    0xCA11,  0xFAE5,  0x1328,  0xFFFD,  0x203C,  0x0003,  0x0003,  0x0200,
    0x0003,  0xFF5C,  0xFF3E,  0x003B,  0xD5AF,  0x0000,  0x0065,  0xFE4A,
    0x0B6F,  0x0057,  0x02A5,  0x0000,  0x01E1,  0xFFF8,  0x3370,  0x0019,
    0xFFAA,  0xFFF3,  0xE0D5,  0x0177,  0x0000,  0xFFD2,  0xFFE1,  0x0011,
    0xFF0D,  0x0173,  0xF924,  0x004B,  0x0012,  0x0E1D,  0xFFFE,  0xFF1A,
    0x03C9,  0x003E,  0x0000,  0xFDE0,  0x0000,  0x0004,  0x0E1B,  0x005B,
    0xFF8E,  0x0017,  0xFA51,  0x02D0,  0xFFDA,  0xFFF1,  0xFFF7,  0xFFFF,
    0xFEFF,  0x39C2,  0x0000,  0xE2F3,  0x1BEF,  0x0002,  0x0056,  0xFFF4,
    0x029C,  0x0012,  0xCE2B,  0xFF6C,  0x0037,  0xFA03,  0x05D5,  0x0008,
    0xFFDD,  0x0000,  0x000C,  0x0220,  0xFDB3,  0xF5DD,  0xFFF1,  0xFFB2,
    0x000E,  0xFFFF,  0xFFD6,  0xFF96,  0xFFFE,  0xFFFE,  0xFE59,  0xC0EB,
    0x19C0,  0xFFEC,  0xFFF8,  0xFFF9,  0xFFFE,  0x0000,  0x0026,  0xDB9D,
    0x0004,  0x15F2,  0xFFC3,  0xFFC4,  0x29B3,  0xFFFB,  0xFFF6,  0xFFDE,
    0xFD6B,  0xC829,  0x0250,  0xE604,  0xFFFF,  0xFF37,  0xF3BE,  0x005C,
    0x5386,  0xFE90,  0xFF19,  0xFFF8,  0x0B38,  0x00D5,  0x24EE,  0x044D,
    0xFFB2,  0xFFEC,  0x0D0D,  0xFFE9,  0xFDCD,  0xFFFE,  0x02A5,  0xFFFF,
    0x0001,  0xFE70,  0xFFFF,  0xFDBE,  0xFE5D,  0x0FF1,  0x0095,  0xFC6F,
    0x000C,  0xEDC1,  0x8AB1,  0xC5EB,  0xFFFF,  0x0354,  0x009F,  0x0000,
    0xFF6E,  0x000E,  0x00CF,  0x0005,  0xFFFF,  0xFFFB,  0x0002,  0xF36D,
    0xFFFE,  0x12D8,  0x0388,  0x00EC,  0x0003,  0x2D66,  0xFA50,  0xFC80,
    0xFFFD,  0xEA26,  0xEBB7,  0x0261,  0x0000,  0x0009,  0x0001,  0x0468,
    0xFFFD,  0x3526,  0x022C,  0x0227,  0xFFFD,  0x0051,  0x0008,  0x6F38,
    0xFFFE,  0x0001,  0xFF21,  0xA428,  0x0000,  0x239A,  0x0000,  0x002D,
    0x00EB,  0x000A,  0xFDE0,  0x0004,  0xFFFE,  0xF22A,  0xFFFF,  0x0000,
    0x0001,  0x0001,  0xF700,  0xFFBD,  0x16EB,  0x0007,  0xFC28,  0xF677,
    0xFF55,  0xFFEA,  0x0000,  0xFC6C,  0xFFF9,  0xFFC4,  0xFFFE,  0xE6ED,
    0x1CE0,  0x011C,  0x0035,  0xFFFE,  0x0007,  0xFFAA,  0xFFF4,  0x0FBF,
    0x3516,  0xE8EE,  0xFFFB,  0x3CC5,  0x30FB,  0x0001,  0x0000,  0xFF63,
    0x0051,  0xFFFE,  0xF8E4,  0xFFFF,  0xF9A2,  0xB968,  0x019F,  0xFD74,
    0xFFEC,  0xF888,  0xF966,  0x036D,  0xFFF8,  0xFFC0,  0xF64B,  0xFFFF,
    0xFFA5,  0x0012,  0x0E31,  0x0056,  0xF89C,  0xFFF0,  0x01F3,  0x0005,
    0x0001,  0xFFE6,  0xFEB4,  0x000E,  0xF062,  0x01DC,  0x05F2,  0xFC43,
    0x00E5,  0xFD5A,  0xF0C6,  0x0221,  0x070C,  0xFFFE,  0xFFC8,  0xFFFF,
    0x0029,  0x01F2,  0xFE19,  0xFFE8,  0xFFFF,  0x0000,  0xFFE9,  0x003C,
    0xFFFD,  0xFCBA,  0xFEAA,  0xFBD9,  0xE47B,  0x0174,  0xFFBE,  0xE13D,
    0xFFFF,  0x001D,  0x0014,  0x49CB,  0xFFFB,  0x40A4,  0x0813,  0x033E,
    0xA69F,  0x3D97,  0x0667,  0xFFFF,  0xFD9D,  0x00F4,  0x0069,  0xFDB3,
    0xFFC5,  0xE5B1,  0x00AF,  0x54EB,  0xFFF4,  0x0001,  0x10EB,  0xDB1B,
    0xFFFE,  0xFFAD,  0x005E,  0x76A4,  0xFFFC,  0x0031,  0x001B,  0xFFE0,
    0x06D2,  0x0278,  0xFFE0,  0xFE11,  0xFFF1,  0x001C,  0x1B38,  0xFFFE,
    0x0000,  0xFFEE,  0xFFFF,  0xFF92,  0x2C96,  0x066D,  0x1BAF,  0x04BD,
    0xFFDD,  0x060C,  0x02D1,  0xFF0A,  0x0A69,  0xFFFF,  0xF8CC,  0xFF6E,
    0xFE64,  0x001F,  0xFD79,  0x0000,  0xFFED,  0x05CF,  0x0001,  0x002A,
    0xFF4F,  0x00A9,  0xF98B,  0xFFF7,  0xFFFC,  0x0348,  0xDEF2,  0x0053,
    0xFFDF,  0x0005,  0x0000,  0x0351,  0x001E,  0xFFAB,  0x0033,  0xFFFA,
    0x0007,  0xFFA8,  0xD1E2,  0xE116,  0x1486,  0xFF40,  0x1A3B,  0x0004,
    0x00A5,  0x0018,  0x0A5D,  0xFFCD,  0xFFFD,  0x0005,  0xFF34,  0xFFFE,
    0xDB6D,  0x001F,  0x044B,  0xFFCC,  0xFCB1,  0x574D,  0xFFFE,  0x0000,
    0x019E,  0xF35A,  0x3A3B,  0x0B5D,  0x218B,  0x0001,  0xFFC3,  0xF716,
    0xFFFF,  0x03CB,  0x0029,  0xFFF9,  0xFDF5,  0x05BB,  0x020D,  0xF844,
    0xC8F0,  0x0971,  0xF198,  0x0007,  0xFFF8,  0xFD64,  0xAE6C,  0x20A3,
    0x1899,  0xFDDE,  0xFFF7,  0x1A24,  0xFFFF,  0xFFFD,  0x0189,  0xD62C,
    0x0164,  0xF6FC,  0xFFEF,  0xFD6A,  0x4D30,  0xFCF7,  0xF612,  0x0012,
    0x0000,  0xFEB6,  0x02AF,  0x0000,  0xFFCD,  0xFFF6,  0xF71F,  0xF881,
    0x0001,  0x0004,  0xFF9F,  0x000D,  0x0001,  0x06F2,  0x0000,  0xFFF5,
    0x016F,  0xFFFF,  0x062F,  0xFC46,  0xEFEA,  0xF91C,  0x0002,  0x0003,
    0xD301,  0x05BC,  0xFFFE,  0xFFF2,  0x0003,  0x3825,  0xF4A2,  0xFFFE,
    0xF869,  0xFFFF,  0xFF96,  0xFFB5,  0xFB42,  0xEFA9,  0xFFFF,  0x0000,
    0xFFFC,  0x1076,  0x0000,  0xF05D,  0x0066,  0xF5EF,  0x0000,  0xFFF7,
    0xFFFF,  0xFFF3,  0xFFFE,  0xF847,  0xF702,  0x0010,  0xFFFB,  0xFE64,
    0xFFF3,  0x0000,  0x03F6,  0xE8F6,  0xFFFE,  0x0000,  0xFFE1,  0xFFFF,
    0xFFFF,  0x0120,  0x0000,  0x0000,  0xB852,  0x0250,  0xFFFF,  0xFFCC,
    0xFFFC,  0x1542,  0xFFFE,  0x001D,  0xFFF0,  0xFFFE,  0xFFE8,  0xF034,
    0xFFFE,  0x0009,  0xFFE1,  0xECA5,  0x0097,  0x0000,  0x0004,  0x509D,
    0x0000,  0xFE2D,  0x0C1B,  0x024D,  0xFD38,  0xFFFF,  0x1CB9,  0xFFFF,
    0xFB5D,  0xFFFF,  0x003A,  0x0008,  0xFF3C,  0xFF99,  0x0028,  0xFFF9,
    0x03A7,  0x0002,  0x0000,  0x0000,  0x0003,  0x0130,  0x1F4C,  0x0F79,
    0x0003,  0xD43D,  0xFF44,  0x0001,  0xFF9C,  0xFF57,  0xFFFE,  0x01E3,
    0x16F8,  0x0001,  0x1C9B,  0x037C,  0xFFFE,  0x0048,  0xFFFB,  0xF5BF,
    0x01F2,  0x049E,  0x002B,  0x06F9,  0x02A6,  0xCF11,  0xF8C5,  0xFFFB,
    0xAF6A,  0x3EAD,  0xFFFD,  0x007F,  0xFFFC,  0x0004,  0x0000,  0xE00A,
    0xFFC5,  0xB899,  0x0001,  0xFF49,  0x0187,  0x0E52,  0x0000,  0x0000,
    0x0000,  0xFFED,  0x005E,  0x13E9,  0x0655,  0x053E,  0x0EE7,  0xFFFF,
    0x2C3F,  0x0000,  0xA510,  0x95FC,  0x0573,  0x5DAC,  0xFBAF,  0xFFF6,
    0xFFF5,  0x0686,  0xFE2E,  0x0000,  0x01BD,  0x0000,  0xFFFA,  0x0A07,
    0x0B7A,  0x0000,  0x000D,  0xFB87,  0xCCC4,  0xFFEB,  0xE5CF,  0x007C,
    0x001E,  0x0008,  0x0000,  0x065D,  0xE5F9,  0xF073,  0xFFF6,  0xFFF3,
    0x0162,  0xFFF9,  0x00B9,  0xD8AC,  0x000B,  0xFE97,  0xFFFF,  0x0004,
    0x0000,  0x01DE,  0x0000,  0xFFA3,  0x0000,  0x0875,  0x0000,  0x1F32,
    0x0000,  0x0049,  0x0000,  0xFFA0,  0x00B4,  0x0002,  0xF911,  0xF8BA,
    0xFB64,  0xFE3C,  0xFFD9,  0xFB1C,  0xFFDE,  0xFFE0,  0x01F5,  0xF614,
    0x0013,  0x0005,  0x091D,  0xFFFF,  0x01F9,  0xFFFD,  0xFB5D,  0x0043,
    0x0351,  0xF865,  0x38EE,  0x0001,  0x0110,  0x0000,  0x0E38,  0x0033,
    0xFF4F,  0x0004,  0x085C,  0xFF14,  0x0B1D,  0x0026,  0x029E,  0xFC4B,
    0xFFC8,  0x01B4,  0x0003,  0xD99A,  0xFD17,  0x05D8,  0xFD89,  0xFFE6,
    0x0000,  0xDD50,  0xDEA6,  0x0009,  0x0000,  0xFEA6,  0x003F,  0xFFED,
    0xF48E,  0xFFFA,  0xFFD2,  0xFA3C,  0xF968,  0x09E8,  0x5993,  0xFF9F,
    0xFFED,  0xF158,  0x0003,  0x0003,  0x0008,  0xFFF4,  0x7A88,  0x017C,
    0xFFCA,  0xFFFC,  0xFFFD,  0xCBAF,  0x0017,  0x3956,  0x0000,  0x0001,
    0xFFFF,  0xFF62,  0x0000,  0xBCF5,  0x3ACC,  0xFC21,  0x4D8E,  0xFFFE,
    0x0310,  0x2723,  0x45FA,  0x0062,  0xFFFF,  0x0050,  0x0072,  0xFFFF,
    0x1CB7,  0xFF68,  0x003D,  0xFF9D,  0xFF1C,  0x001F,  0xFFFF,  0x0002,
    0xFFFE,  0xFFFF,  0x00FB,  0xFFB7,  0xF421,  0x0010,  0xFFE5,  0x0012,
    0xFFFF,  0x02E7,  0x0000,  0xFFEE,  0xFFFA,  0x0015,  0x2792,  0xFFDC,
    0xFFFC,  0xFFFF,  0xFFFF,  0xFD1C,  0xFCFC,  0xFDB6,  0x000B,  0x0006,
    0x0613,  0xFFCA,  0xFFEB,  0xFFFE,  0xFD94,  0xCDC5,  0xFFF0,  0xFFE6,
    0x0004,  0x0003,  0x0001,  0x005A,  0x0002,  0x0001,  0xFFD0,  0xFE98,
    0x0694,  0x0000,  0xFFF8,  0x88FA,  0xFFFC,  0xF8D5,  0xFC47,  0xFF38,
    0x0012,  0xFFFF,  0x0000,  0x0789,  0xFF18,  0xECF3,  0xF4A2,  0x0027,
    0xFFB1,  0x0314,  0xFFFE,  0x001A,  0xFDB7,  0xFDBD,  0xFE1C,  0x0003,
    0x29B3,  0xD5FB,  0xFE7F,  0xFC92,  0xF745,  0xF9B5,  0xFEC4,  0x6DE2,
    0xFFFD,  0x23DE,  0xFDC9,  0x000D,  0x000B,  0xFFE4,  0x000D,  0xFB43,
    0x0026,  0x006E,  0x0B12,  0xFFFD,  0xFFFE,  0x0007,  0x025D,  0xFFFF,
    0x1EC6,  0xFE14,  0x0042,  0x0371,  0xFFFF,  0x0009,  0xA8A6,  0xF9DA,
    0xFF4F,  0xFFCC,  0x1170,  0xFFFF,  0x01BA,  0xFDC0,  0x0366,  0xFFFE,
    0x1875,  0x0005,  0xFEDB,  0xFC34,  0xFFDA,  0xFFF4,  0xFE9A,  0x3E36,
    0xFFFE,  0xD51F,  0xFFBE,  0xFF1B,  0x0964,  0x0009,  0xFFCF,  0xF37E,
    0xFF6A,  0xFFE3,  0xFFFF,  0x0005,  0xF9A5,  0x07A3,  0x0001,  0xF421,
    0xFFEC,  0xFFBB,  0xFFFE,  0x001B,  0xFE8B,  0xFFFA,  0x0023,  0x3271,
    0x0000,  0xFFFB,  0x0009,  0xFFFC,  0xFFFC,  0x0A58,  0x0000,  0x0044,
    0xD658,  0x0013,  0xFFFE,  0x0000,  0x01F1,  0x00BF,  0x0683,  0x31A6,
    0x0004,  0x0005,  0xF303,  0x0104,  0xFD1A,  0xEA53,  0x0098,  0xFFFF,
    0xFFFD,  0xFFFE,  0xFFF1,  0xFF9B,  0xFE88,  0xFFA0,  0xFFFF,  0x0016,
    0x0000,  0xFFCE,  0xFFFB,  0xB4B5,  0x0005,  0x57D9,  0x0007,  0x0003,
    0x0035,  0x0005,  0xCAAB,  0xF410,  0xE9A1,  0xFE26,  0xFFD8,  0xFFFF,
    0xFFFE,  0xFFFD,  0xFFF3,  0x283E,  0x25AD,  0x0002,  0x15B6,  0x78F2,
    0xFFCF,  0xF44B,  0xFFFF,  0x0000,  0xFF97,  0x0018,  0x0035,  0xFFF7,
    0x0011,  0xFFA1,  0xFFFF,  0xFF6F,  0x065C,  0x002D,  0xFFFB,  0xFFFE,
    0x0010,  0xFF66,  0x84EE,  0x28FF,  0xFFAA,  0xFE84,  0x01CD,  0x000D,
    0xFACC,  0x0020,  0x00B6,  0xFFFF,  0xF01F,  0x002B,  0xF0ED,  0xFFFC,
    0xFFF6,  0x02E1,  0x0040,  0x001F,  0x0061,  0xFF75,  0xAB5F,  0x0773,
    0xF6E4,  0x0011,  0xFFFE,  0xCAE8,  0x0104,  0xFFCE,  0x0004,  0x0323,
    0xFFFE,  0x00AF,  0xFE3D,  0xC32B,  0xFF04,  0x0015,  0xFBC6,  0x348C,
    0xFFC4,  0x1937,  0x37BB,  0x07F2,  0xFFF9,  0x103F,  0x0000,  0xFFD1,
    0x000A,  0xFFCC,  0x5514,  0x0906,  0xF6E8,  0x0023,  0x0000,  0xFFFB,
    0x0000,  0xF8EE,  0x0212,  0xFFFF,  0x002B,  0x03B4,  0xFFF2,  0xFFFF,
    0x0BF8,  0xFCF9,  0xFFF0,  0x0000,  0x0046,  0x0016,  0x12E8,  0xC8D4,
    0x007A,  0xFE8A,  0xFFFF,  0xFFF9,  0x0365,  0xFE4E,  0x01A7,  0x2C8D,
    0xFFFF,  0xFFFF,  0xFFFF,  0xFFFB,  0xFDE4,  0x0007,  0x0817,  0x0163,
    0x056D,  0x0000,  0x0006,  0x4692,  0xFFE9,  0xFBC3,  0x072C,  0xFFFF,
    0x001B,  0xFFF9,  0x0007,  0xFFFC,  0xFFFA,  0x0002,  0x0002,  0xFFFF,
    0x0038,  0x8BFF,  0xA286,  0x046B,  0x0000,  0xE060,  0xFEEF,  0xF94C,
    0x0000,  0x03EC,  0x0007,  0xFEC0,  0x5A94,  0xFFC0,  0x00AC,  0x0188,
    0xFE46,  0xFE81,  0xFFF4,  0x07CF,  0xF28A,  0x02EA,  0xFFFF,  0x00F5,
    0xFFE5,  0xF0D1,  0xFF99,  0x000A,  0x000F,  0x09CF,  0xFFFD,  0x04C0,
    0x0002,  0x0079,  0x03C0,  0x0002,  0xFFC7,  0x04B8,  0x0015,  0xFD2C,
    0xF848,  0xFFFE,  0x0002,  0xFFF8,  0x1DFF,  0x04B9,  0x175F,  0x000B,
    0x0000,  0xFFF6,  0x0306,  0x0000,  0x0003,  0x0001,  0x0E76,  0x002D,
    0x0001,  0xCF4F,  0x30A2,  0x0007,  0xFF89,  0xFFFE,  0xFFFC,  0x0000,
    0xFFFE,  0xFFFB,  0xFFD7,  0xFFE0,  0xFFFD,  0xFFFF,  0x1A3A,  0xFFFA,
    0xFFFF,  0xFFF7,  0x0950,  0x0003,  0xFFFD,  0x000F,  0x0000,  0xAD6B,
    0x0073,  0xFF21,  0xFFCF,  0x0007,  0xE0D3,  0x0000,  0xFF65,  0x00F7,
    0x0016,  0x00CE,  0xFFDB,  0x0464,  0xFFF4,  0xFFFD,  0xFFC7,  0x0015,
    0x6726,  0xFFEA,  0x0005,  0xFE40,  0x000F,  0x0000,  0xFF86,  0x03E9,
    0xFFFE,  0xFFFB,  0x354F,  0xFFFD,  0x0000,  0x000C,  0x00F0,  0xFFFE,
    0xFE0C,  0xCA1A,  0x26FC,  0x0002,  0xFDF0,  0x0000,  0xFF93,  0xF082,
    0x1041,  0xF7B9,  0x01BD,  0x0BAA,  0x05D1,  0xFF84,  0x007F,  0xFC31,
    0xFFFF,  0x0001,  0x000B,  0xFFC7,  0xFF8F,  0x0003,  0xCDEC,  0xFFD2,
    0xB1EA,  0xFFF5,  0xFFF6,  0xFFFC,  0xE29B,  0x0007,  0x0000,  0x16C9,
    0xFC31,  0xFFFC,  0x0261,  0xFFFF,  0x1AEA,  0x00DE,  0xFC1E,  0xFFE4,
    0xFF28,  0xFE59,  0xFFFE,  0xFFE9,  0xFFFD,  0x0001,  0x0120,  0xFFFD,
    0x0000,  0xFFFC,  0xFE4F,  0xFFFF,  0x0A75,  0x096E,  0xFF93,  0xFF62,
    0xFFFF,  0xFE3E,  0xFFF1,  0xFF34,  0xFFF3,  0xF75C,  0xFFFE,  0x0001,
    0xFC7F,  0x0025,  0x0007,  0x0753,  0x00A7,  0xFFE1,  0xFD17,  0xFF90,
    0xFA43,  0x0000,  0xFFFF,  0xFF9F,  0xFFF9,  0x002F,  0x73FF,  0x0641,

    0xFFAA,  0x5AA3,  0x765A,  0x5AA5,  0xCDEA,  0x7BCD,  0xFFDC,  0x7FFF,      // Ending
    0xFEAD,  0xDADE,  0xCEAD,  0xFDAC,  0xAAAA,  0xA5A5,  0x5A5A,  0x55AA       // PAD
};

const short errCoefs[NerrCoefs + 2*PAD] =
{
    0xFFAA,  0x5AA3,  0x765A,  0x5AA5,  0xCDEA,  0x7BCD,  0xFFDC,  0x7FFF,      // Beginning
    0xFEAD,  0xDADE,  0xCEAD,  0xFDAC,  0xAAAA,  0xA5A5,  0x5A5A,  0x55AA,      // PAD

    0xFC7F,  0x0753,  0x069F,  0xFFFD,  0x1A29,
    0xEAEA,  0xAEAE,  0xAAEE,  0xFFF6,  0xFFF2,

    0xFB9E,  0x0FD6,  0xFB9E,  0xBABA,  0xABAB,  0xAABB,  0xFFAA,  0x5AA3,      // Ending
    0x765A,  0x5AA5,  0xCDEA,  0x7BCD,  0xFFDC,  0x7FFF,  0xFEAD,  0xDADE       // PAD
};

int max_index_c, max_index_i;
int ret_value_c, ret_value_i;

/* ======================================================================= */
/* Prototypes for timing functions                                         */
/* ======================================================================= */
clock_t time_c();
clock_t time_i();

/* ======================================================================= */
/* MAIN -- Top level driver for testing the algorithm                      */
/* ======================================================================= */
int main()
{
    clock_t t_overhead, t_start, t_stop;
    clock_t t_c, t_i;
    int fail = 0;
    int form_error = 0;

    /* =================================================================== */
    /* Initialize timer for clock */
    TSCL= 0,TSCH=0;
    /* Compute overhead of calling _itoll(TSCH, TSCL) twice to get timing info        */
    /* =================================================================== */
    t_start = _itoll(TSCH, TSCL);
    t_stop = _itoll(TSCH, TSCL);
    t_overhead = t_stop - t_start;

    /* =================================================================== */
    /* Call the individual timing routines, and subtract off overhead      */
    /* =================================================================== */
    t_c = time_c() - t_overhead;    // Calculate time to run Natural C Algorithm
    t_i = time_i() - t_overhead;    // Calculate time to run Optimized C Algorithm

    /* =================================================================== */
    /* Print timing results                                                */
    /* =================================================================== */
    printf("DSP_minerror\t");

    /* =================================================================== */
    /* Check the results arrays, and report any failures                   */
    /* =================================================================== */
    if (memcmp(&max_index_i, &max_index_c, sizeof(max_index_c))) {
        fail++;
        printf("Result Failure (max_index_i)\t");
    }
    else
        printf("Result Successful (max_index_i)\t");
    if (memcmp(&ret_value_i, &ret_value_c, sizeof(ret_value_c))) {
        fail++;
        printf("Result Failure (ret_value_i)");
    }
    else
        printf("Result Successful (ret_value_i)");

    printf("\tnatC: %d\toptC: %d\n", t_c, t_i);

    /* Provide memory information */
#ifdef __TI_COMPILER_VERSION__            // for TI compiler only
    printf("Memory:  %d bytes\n", &kernel_size);
#endif

    /* Provide profiling information */
    printf("Cycles:  %d\n", t_i);

    return(fail);
}

/* ======================================================================= */
/* Prototypes for timing functions                                         */
/* ======================================================================= */
clock_t time_c()
{
    clock_t t_start, t_stop;

    t_start = _itoll(TSCH, TSCL);
    ret_value_c = DSP_minerror_cn(GSP0_TABLE + PAD, errCoefs + PAD, &max_index_c);
    t_stop = _itoll(TSCH, TSCL);
    return t_stop - t_start;
}

/* ======================================================================= */
/* Prototypes for timing functions                                         */
/* ======================================================================= */
clock_t time_i()
{
    clock_t t_start, t_stop;

    t_start = _itoll(TSCH, TSCL);
    ret_value_i = DSP_minerror(GSP0_TABLE + PAD, errCoefs + PAD, &max_index_i);
    t_stop = _itoll(TSCH, TSCL);
    return t_stop - t_start;
}

/* ======================================================================= */
/*  End of file:  DSP_minerror_d.c                                         */
/* ----------------------------------------------------------------------- */
/*            Copyright (c) 2007 Texas Instruments, Incorporated.          */
/*                           All Rights Reserved.                          */
/* ======================================================================= */

